/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008 <talsi.elderme@gmail.com>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



uniform vec3 skyColor;
uniform vec3 groundColor;

varying vec3 hemiDir;

void main (void) {

	//eye vector
	vec3 vVec = normalize(eyeDir);

	//texture
	vec2 texCoord = gl_TexCoord[0].st;

	//normal
	vec3 normal;

	//colors
	vec4 emiColor, ambColor, diffColor, specColor;
 	material (texCoord, normal, emiColor, ambColor, diffColor, specColor);

	//light direction, attenuation and distance
	vec3 lVec;
	vec4 attAndDist;

	//light color coefficients
	vec4 ambLighting = vec4(0.);
	vec4 diffLighting = vec4(0.);
	vec4 specLighting = vec4(0.);

	specColor.a = specColor.a * 42.;
	lightingCoeff(lightDir, d, vVec, normal, specColor.a, ambLighting, diffLighting, specLighting);

	//hemisphere ambient lighting
	float cosTheta = dot(normal,normalize(hemiDir));
	//float cosTheta = dot(vec3(0.,0.,1.),normalize(hemiDir));
	float a = 0.5 + 0.5 * cosTheta;
	vec3 ambientModel = mix(groundColor, skyColor, a);

	vec3 color = ambientModel + (emiColor + ambColor*ambLighting + diffColor*diffLighting + specColor*specLighting).rgb;

	//vec3 color = vec3(0.04, 0.08, 0.1) + (emiColor + ambColor*ambLighting + diffColor*diffLighting + specColor*specLighting).rgb;
	//vec3 color = ambientModel;
	//vec3 color = (diffColor*diffLighting).rgb;
	//vec3 color = (ambColor*ambLighting).rgb;
	//vec3 color = (specColor*specLighting).rgb;

	if(color.r>1.0) color.r=1.0;
	if(color.g>1.0) color.g=1.0;
	if(color.b>1.0) color.b=1.0;

	//fog
	float fog = (gl_Fog.end - gl_FogFragCoord) * gl_Fog.scale;
	fog = clamp(fog, 0., 1.);
	color = mix(vec3(gl_Fog.color), color, fog);

	gl_FragColor = vec4(color, diffColor.a);


}



