/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "entityPlane.h"
	
EntityPlane::EntityPlane(int size, float interval, bool visible,
				Vector3 axis1, Vector3 axis2, 
				Vector3 center, Vector3 rotAxis, float angle) :
	EntityWithPosition(NULL, NULL, false, center, rotAxis, angle), mSize(size), mInterval(interval), mAxis1(axis1), mAxis2(axis2) {
	
}

EntityPlane::EntityPlane(int size, float interval, bool visible, Material *material, Mesh *mesh,
						 Vector3 axis1, Vector3 axis2, 
				  	  	 Vector3 center, Vector3 rotAxis, float angle):
	EntityWithPosition(NULL, NULL, false, center, rotAxis, angle), mSize(size), mInterval(interval), mAxis1(axis1), mAxis2(axis2) {
		
	for(int i=0;i<mSize;i++) {
		for(int j=0;j<mSize;j++) {
			addChild(new EntityWithValueInPlane (material, mesh, 0, i, j, interval, visible, axis1, axis2), i*mSize + j);
		}		
	}
}

EntityPlane::~EntityPlane() {
}
	

void EntityPlane::insertChild(Material *material, Mesh *mesh, int value, int position1, int position2) {
	addChild(new EntityWithValueInPlane (material, mesh, value, position1, position2, mInterval, true, mAxis1, mAxis2), position1*mSize + position2);
}

int EntityPlane::getValue(int i, int j) {
	
	int position = i*mSize + j;
	
	if(children.find(position) != children.end()) {
		EntityWithValueInPlane *entity = dynamic_cast <EntityWithValueInPlane*> (children[position]);
		return entity->getValue();	
	} else {
		return -1;
	}	
}

int EntityPlane::getValue(int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInPlane *entity = dynamic_cast <EntityWithValueInPlane*> (children[position]);
		return entity->getValue();	
	} else {
		return -1;
	}	
}


bool EntityPlane::setValue(int value, int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInPlane *entity = dynamic_cast <EntityWithValueInPlane*> (children[position]);
		entity->setValue(value);
		return true;
	} else {
		return false;
	}
}

bool EntityPlane::setValueAndMaterial(int value, Material *material, int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInPlane *entity = dynamic_cast <EntityWithValueInPlane*> (children[position]);
		entity->setValue(value);
		entity->setMaterial(material);
		return true;
	} else {
		return false;
	}
}

void EntityPlane::setValueAndMaterialForAllEntities(int value, Material *material) {
	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		EntityWithValueInPlane *entity = dynamic_cast <EntityWithValueInPlane*> ((*it).second);
		entity->setValue(value);
		entity->setMaterial(material);
	}
}



void EntityPlane::setClickable() {
	mNumClick = -1;

	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		Entity *child = (*it).second;
		int index = (*it).first;
		child->setNumClick(index);
	}	
}
