/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "glUtil.h"
  
bool VBOsupported = 0;
bool GLSLsupported = 0;
bool FBOsupported = 0;




bool isExtensionAvailable(char *extensionName) {
	std::string Extensions = reinterpret_cast<const char*>(glGetString(GL_EXTENSIONS));

	if (Extensions.find(extensionName) != std::string::npos) {
		return true;
	} else {
		std::cout << extensionName << " is not available." << std::endl;
		return false;
	}		
} 
 
void printGLInfos() {
	std::cout << "OpenGL renderer string: " << glGetString(GL_RENDERER) << std::endl;
	std::cout << "OpenGL vendor string: " << glGetString(GL_VENDOR) << std::endl;
	std::cout << "0penGL version string: " << glGetString(GL_VERSION) << "\n" << std::endl;
	
	int nb;
	GLboolean b;
	GLfloat f;
	glGetIntegerv (GL_MAX_LIGHTS,&nb);
	std::cout << "Maximum number of lights: " << nb << std::endl;
	glGetIntegerv (GL_MAX_NAME_STACK_DEPTH, &nb);
	std::cout << "Maximum depth of name stack: " << nb << std::endl;
	glGetIntegerv (GL_AUX_BUFFERS,&nb);
	std::cout << "Number of aux buffers: " << nb << std::endl;
	glGetBooleanv ( GL_STEREO,&b);
	if(b==GL_TRUE)
		std::cout << "You have a right and a left buffer." << std::endl;
	glGetIntegerv (GL_MAX_TEXTURE_UNITS_ARB,&nb);
	std::cout << "Maximum number of texture units: " << nb << std::endl;
	glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &f);
	std::cout << "Maximum number for anisotropic filtering: " << f <<std::endl;
	glGetIntegerv (GL_MAX_TEXTURE_SIZE, &nb);
	std::cout << "Maximum texture size: " << nb << std::endl;
}

void printGLSLInfos() {
	int nb;
	
	std::cout << "GLSL version: " << glGetString(GL_SHADING_LANGUAGE_VERSION) << std::endl;
	
	glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS, &nb);
	std::cout << "Maximum number of texture image units: " << nb << std::endl;

	glGetIntegerv (GL_MAX_DRAW_BUFFERS, &nb);
	std::cout << "Maximum number of buffers that can be simultaneously written into: " << nb << std::endl;

	glGetIntegerv (GL_MAX_FRAGMENT_UNIFORM_COMPONENTS, &nb);
	std::cout << "Maximum number of uniform components available for a fragment shader: " << nb << std::endl;
	
	glGetIntegerv (GL_MAX_TEXTURE_COORDS, &nb);
	std::cout << "Maximum number of texture coordinate sets: " << nb << std::endl;
	
	glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS, &nb);
	std::cout << "Maximum number of hardware units that can be use for texture maps: " << nb << std::endl;
				   
	glGetIntegerv (GL_MAX_VARYING_FLOATS, &nb);
	std::cout << "Maximum number of floating-point variables available for varying variables: " << nb << std::endl;
				   
	glGetIntegerv (GL_MAX_VERTEX_ATTRIBS, &nb);
	std::cout << "Maximum number of vertex attributes: " << nb << std::endl;
				   
	glGetIntegerv (GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS, &nb);
	std::cout << "Maximum number of hardware units that cans be used for texture mapping form the vertex processor: " << nb << std::endl;
				   
	glGetIntegerv (GL_MAX_VERTEX_UNIFORM_COMPONENTS,  &nb);
	std::cout << "Maximum number of components that are available for vertex shader uniform variables: " << nb << std::endl;
	
}

void printGLError() {
	GLenum errCode;
	const GLubyte *errString;
	
	if((errCode = glGetError ()) != GL_NO_ERROR) {
		errString = gluErrorString (errCode);
		std::cout << "OpenGL error: " << errString << std::endl;
	}
}



int setAnisotropicFiltering(int nbAnisotropy) {
	//window_loadGLextension("GL_EXT_texture_filter_anisotropic");

	/*GLfloat f;
	f = 8.;
	glTexParameterfv (GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, &f);*/
	
	glTexParameteriv (GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, &nbAnisotropy);
	
	GLfloat ff;
	glGetTexParameterfv (GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, &ff);
	printGLError();

	std::cout << "Anisotropic filtering: " << ff <<std::endl;
	

}

int getMaxLights() {
	int nb;
	glGetIntegerv (GL_MAX_LIGHTS,&nb);
	return nb;
}

int getMaxTextureUnits() {
	int nb;	
	glGetIntegerv (GL_MAX_TEXTURE_UNITS_ARB,&nb);
	return nb;	
}

int getTextureSizeMax() {
	int nb;		
	glGetIntegerv (GL_MAX_TEXTURE_SIZE, &nb);
	return nb;	
}



int processHits(GLint hits, GLuint buffer[]){
	int i,j;
	GLuint names, *ptr;
	
	int closest = -1;
	float zClosest = 4200.;
	float z1,z2;	
	
	ptr = (GLuint *) buffer;
	for(i=0;i<hits;i++) {		
		names = *ptr; //it will usually be 1
		ptr++;
		z1 = (float) *ptr/0x7fffffff;
		ptr++;
		//z2 = (float) *ptr/0x7fffffff;
		ptr++;
		
		if(z1<zClosest){
			zClosest = z1;
			closest = *ptr;
		}
		for(j=0;j<names;j++){
			ptr++;
		}
	}
	
	return closest;
}

std::vector<int> processHitsList(int nbObjects, GLint hits, GLuint buffer[]) {
	std::vector<int> res;
	
	int i,j;
	GLuint names, *ptr;
	
	int closest;
	float zClosest = 42.;
	float z1,z2;	
	
	ptr = (GLuint *) buffer;
	for(i=0;i<hits;i++) {
		names = *ptr; //it will usually be 1
		ptr++;
		z1 = (float) *ptr/0x7fffffff;
		ptr++;
		//z2 = (float) *ptr/0x7fffffff;
		ptr++;
		if(z1<zClosest){
			zClosest = z1;
			closest = *ptr;
		}
		for(j=0;j<names;j++){
			ptr++;
		}
	}
	
	if(hits>0){
	
	}
		
	return res;
}

