/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "sky.h"

Sky::Sky(std::string skyTexFilename, std::string skyNightTexFilename): mSunDirection(0.,1.,0.) {
	ImagePng *sky = new ImagePng(skyTexFilename,0);
	ImagePng *skyNight = new ImagePng(skyNightTexFilename,0);
	
	skyTex = new TextureLayer(sky, OTHER_MAP, GL_CLAMP_TO_EDGE, false);
	delete sky;
	
	skyNightTex = new TextureLayer(skyNight, OTHER_MAP, GL_CLAMP_TO_EDGE, false);
	delete skyNight;
	
	//float angle = 8.*M_PI/17.;
	//float angle = 4.*M_PI/9.;
	//float angle = 2.*M_PI/5.;
	float angle = M_PI/4.;
	//float angle = 0.;
	//float angle = M_PI/2.;
	//float angle = 9.*M_PI/17.;
	
	mSunDirection = Vector3(0.,cos(angle),sin(angle)); 
	
	skyDome = new PartialSphereMesh (300., M_PI /2., 60, 60, 0., true);
	skyDome->initialize(2);
}

Sky::~Sky() {
	delete skyTex;
	delete skyNightTex;	
	delete skyDome;
}


void Sky::setSunDirection (Vector3 sunDirection) {
	mSunDirection = sunDirection;
	shaderMana->getSkyShader()->setSunDirection(mSunDirection);	
}

Vector3 Sky::getSunDirection (Vector3 sunDirection) {
	return mSunDirection;
}


void Sky::display() {
	shaderMana->getSkyShader()->setSunDirection(mSunDirection);
		
	glActiveTexture(GL_TEXTURE0);
	skyTex->activate();
	glActiveTexture(GL_TEXTURE1);
	skyNightTex->activate();
	
	shaderMana->getSkyShader()->setTexId(0,1);	
	skyDome->draw(2);
	
	skyTex->release();
	skyNightTex->release();
	glActiveTexture(GL_TEXTURE0);	
}



/****************************/
/*	SkyPlane				*/
/****************************/

SkyPlane::SkyPlane(std::string skyTexFilename, float dist, float height, bool alphaClouds) : mHeight(height) {
	mpPlane = new RectangleMesh (2.*dist, 10, 2.*dist, 10);
		
	mpMaterial = new Material ("materialSkyPlane", 14);
	mpMaterial->addTexture(skyTexFilename, true, 0, OTHER_MAP, GL_REPEAT, false, NULL);

	if(alphaClouds) {
		mpMaterial->setBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}

	mpPlane->initialize(mpMaterial->getNumberOfTextures());	
}

SkyPlane::~SkyPlane() {
	if(mpMaterial!=NULL) {
		delete mpMaterial;
	}
	if(mpPlane!=NULL) {
		delete mpPlane;
	}
}

void SkyPlane::display () {
	glPushMatrix();
	glTranslatef (0.,mHeight, 0.);	
	glRotatef (90., 1., 0., 0.);
	mpMaterial->activate(0, FIXED_PIPELINE);
	mpPlane->draw(mpMaterial->getNumberOfTextures());
	mpMaterial->release();
	glPopMatrix();	
}






