/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _VECTOR3_H_
#define _VECTOR3_H_

#include <math.h>

class Vector3 {
public:
	Vector3();
	Vector3(float a,float b,float c);
		
	float x, y, z;	
	
	float getNorm();
	float getNorm2();
	float normalize();
	
	/* Generates a vector perpendicular to this vector */
	Vector3 perpendicular();
	/* rotate the vector
	 * axis is the axis of the rotation (must have norm(axis) = 1)
	 * theta is the angle of the rotation */
	Vector3 rotate(Vector3 axis, float theta);
	
	friend Vector3 operator+(const Vector3& vect1,const Vector3& vect2);
	friend Vector3 operator-(const Vector3& vect1,const Vector3& vect2);
	friend Vector3 operator*(const Vector3& vect, const float scalar);
	friend Vector3 operator*(const float scalar, const Vector3& vect);
		
	friend float operator*(const Vector3& vect1,const Vector3& vect2);
	friend Vector3 operator^(const Vector3& vect1,const Vector3& vect2);	

    inline operator float* () {
		return &x;
	}

	inline operator const float* () const {
		return &x;
	}
		
		
};

#endif //_VECTOR3_H_
